@@ -75,18 +75,28 @@ class Clickhouse:
         client.command(f"""ALTER TABLE {table_name} RENAME COLUMN test2 TO {column_name};""")
 
     # датафрейм, название таблицы -> вставка данных
-    def ch_insert(self, df, to_table):
+    import numpy as np
+
+
+
+    def ch_insert(self, df, to_table,chunk_size=20000):
         try:
             data_tuples = [tuple(x) for x in df.to_numpy()]
             client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username,
                                                    password=self.password, database=self.database)
-            client.insert(to_table, data_tuples, column_names=df.columns.tolist())
+
+            for i in range(0, len(df), chunk_size):
+                chunk = df.iloc[i:i + chunk_size]
+                data_tuples = [tuple(x) for x in chunk.to_numpy()]
+                client.insert(to_table, data_tuples, column_names=chunk.columns.tolist())
+                time.sleep(1)
+
             message =f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {to_table}. Результат: данные вставлены в CH!'
             self.common.log_func(self.bot_token, self.chat_list, message, 1)
             optimize_table = f"OPTIMIZE TABLE {to_table} FINAL"
             client.command(optimize_table)
         except Exception as e:
-            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {to_table}. Ошибка вставки в CH: {e}'
+            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {to_table}. Функция: ch_insert. Ошибка: {e}'
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             raise
         finally:
@@ -205,7 +215,7 @@ class Clickhouse:
 
 
 
-    def get_missing_dates(self, table_name, report_name, start_date_str):
+    def get_missing_dates(self, table_name, report_name, start_date_str, include_today):
         try:
             start_date = datetime.strptime(start_date_str, '%Y-%m-%d').date()
             query = f"""
@@ -217,9 +227,14 @@ class Clickhouse:
             existing_dates = {row[0] for row in result.result_rows}
             current_date = start_date
             all_dates = set()
-            while current_date < self.today:
-                all_dates.add(current_date)
-                current_date += timedelta(days=1)
+            if include_today:
+                while current_date <= self.today:
+                    all_dates.add(current_date)
+                    current_date += timedelta(days=1)
+            else:
+                while current_date < self.today:
+                    all_dates.add(current_date)
+                    current_date += timedelta(days=1)
             missing_dates = sorted(all_dates - existing_dates)
             missing_dates_str = [date.strftime('%Y-%m-%d') for date in missing_dates]
             message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {table_name}. Старт: {start_date}. Функция: get_missing_dates. Результат: ОК'
@@ -288,7 +303,7 @@ class Clickhouse:
             self.ch_execute(optimize_collection)
             time.sleep(4)
             if history:
-                date_list = self.get_missing_dates(f'{platform}_collection_{self.add_name}', report_name, self.start)
+                date_list = self.get_missing_dates(f'{platform}_collection_{self.add_name}', report_name, self.start,  False)
                 for date in date_list:
                     if self.err429 == False and self.common.to_collect(frequency, date):
                         message = f'Платформа: {platform}. Имя: {self.add_name}. Таблица: {upload_table}. Репорт: {report_name}. Дата: {date}. Начинаем сбор...'
