@@ -13,7 +13,7 @@ import math
 from transliterate import translit
 
 class Common:
-    def __init__(self, bot_token:str, chat_list:str, message_type: str):
+    def __init__(self, bot_token:str = '', chat_list:str = '', message_type: str = ''):
         self.bot_token = bot_token
         self.chat_list = chat_list
 
@@ -26,6 +26,7 @@ class Common:
         self.now = datetime.now()
         self.today = datetime.now().date()
 
+
     def log_func(self, bot_token, chat_ids,message, value):
         try:
             print(message)
@@ -55,6 +56,23 @@ class Common:
         except Exception as e:
             print(f'Ошибка send_logs_clear: {e}')
 
+    def send_logs_clear_anyway(self,bot_token, chat_ids):
+        try:
+            log_file_path = "/app/log.txt"
+            if not os.path.exists(log_file_path):
+                print("Файл лога не существует.")
+            with open(log_file_path, "r") as log_file:
+                content = log_file.read()
+            if len(content.strip()) > 0:
+                self.message_text = content.strip()
+                self.send_logs(bot_token, chat_ids)
+                with open(log_file_path, "w") as log_file:
+                    log_file.write("")  # Очищаем файл
+                print("Файл очищен, длина содержимого превышала 1000 символов.")
+            return content
+        except Exception as e:
+            print(f'Ошибка send_logs_clear: {e}')
+
     def send_logs(self, bot_token, chat_ids):
         try:
             url = f'https://api.telegram.org/bot{bot_token}/sendMessage'
@@ -84,6 +102,21 @@ class Common:
         except Exception as e:
             print(f'Ошибка flip_date: {e}')
 
+    def datetime_to_unixtime(self, datetime_str):
+        # Парсим строку даты и времени в объект datetime
+        date_obj = datetime.strptime(datetime_str, '%Y-%m-%d %H:%M:%S')
+        # Преобразуем в Unix timestamp (в секундах)
+        timestamp = int(date_obj.timestamp())
+        return timestamp
+
+    def get_month_start(self, date_str: str):
+        try:
+            date_obj = datetime.strptime(date_str, '%Y-%m-%d')
+            month_start = date_obj.replace(day=1)
+            return month_start.strftime('%Y-%m-%d')
+        except Exception as e:
+            print(f'Ошибка get_month_start: {e}')
+
     def is_error(self, result):
         if isinstance(result, str):
             if 'Ошибка:' in result:
@@ -97,8 +130,32 @@ class Common:
 
     def transliterate_key(self, key):
         tr = translit(key, 'ru', reversed=True)
-        tr = tr.strip().replace(' ', '_').replace('-', '_').replace(",", '').replace("'", '').replace(".", '').replace("(",'').replace(")", '').lower()
-        return  tr
+        tr = tr.strip().replace('%','').replace(':','_').replace(' ', '_').replace('-', '_').replace(",", '').replace("'", '').replace(".", '').replace("(",'').replace(")", '').lower().strip()
+        return tr
+
+    def transliterate_dict_keys_in_list(self, dictionaries_list):
+        updated_list = []
+        for dictionary in dictionaries_list:
+            updated_dict = {}
+            for key, value in dictionary.items():
+                new_key = self.transliterate_key(key)
+                updated_dict[new_key] = value
+            updated_list.append(updated_dict)
+        return updated_list
+
+
+    def tuple_none_change(self,my_tuple):
+        my_list = list(my_tuple)
+        counter = 1
+        for i in range(len(my_list)):
+            if my_list[i] is None:
+                my_list[i] = f'None{counter}'
+                counter += 1
+            else:
+                my_list[i] = my_list[i].replace('№','nomer').replace('%','percent').replace('/','slash').replace('\\','slash').replace('(','').replace(')','')
+        new_tuple = tuple(my_list)
+        return new_tuple
+
 
     def replace_keys_in_data(self, dictionaries_list):
         updated_list = []
@@ -136,7 +193,7 @@ class Common:
     def get_data_type(self, column, getvalue, partitions):
         getvalue = str(getvalue)
         part_list = partitions.replace(' ', '').split(',')
-        if getvalue == None or getvalue.strip() == '': return 'None'
+        if getvalue == None or getvalue == 'None' or getvalue.strip() == '': return 'None'
         if getvalue.lower() == 'false' or getvalue.lower() == 'true':
             return 'UInt8'
         date_formats = [
@@ -149,6 +206,8 @@ class Common:
             "%Y-%m-%dT%H:%M:%S",  # 2023-10-22T16:36:15 (без миллисекунд и таймзоны)
             "%Y-%m-%d %H:%M:%S",  # 2023-10-22 16:36:15 (без 'T', без миллисекунд)
             "%Y-%m-%d",  # 2023-10-22 (только дата)
+            "%Y.%m.%d",  # 2023-10-22 (только дата)
+            "%d.%m.%Y",  # 22-10-2023 (европейский формат)  # Формат Date с днем в начале: 08-09-2021
             "%d-%m-%Y",  # 22-10-2023 (европейский формат)  # Формат Date с днем в начале: 08-09-2021
             '%Y/%m/%d',  # Формат Date через слэш: 2024/09/01
             '%H:%M:%S',  # Формат Time: 21:20:10
@@ -160,7 +219,7 @@ class Common:
                 if parsed_date.year < 1970:
                     return 'String'
                 # Определяем тип на основе формата
-                if date_format in ['%Y-%m-%d', '%d-%m-%Y', '%Y/%m/%d']:
+                if date_format in ['%Y-%m-%d', '%d-%m-%Y','%Y.%m.%d', '%d.%m.%Y', '%Y/%m/%d']:
                     return 'Date'  # Это формат Date
                 elif date_format == '%H:%M:%S':
                     return 'Time'  # Это формат Time
@@ -199,7 +258,9 @@ class Common:
             "%Y-%m-%dT%H:%M:%S",  # 2023-10-22T16:36:15 (без миллисекунд и таймзоны)
             "%Y-%m-%d %H:%M:%S",  # 2023-10-22 16:36:15 (без 'T', без миллисекунд)
             "%Y-%m-%d",  # 2023-10-22 (только дата)
-            "%d-%m-%Y"  # 22-10-2023 (европейский формат)
+            "%d-%m-%Y",  # 22-10-2023 (европейский формат)
+            "%Y.%m.%d",  # 2023-10-22 (только дата)
+            "%d.%m.%Y"  # 22-10-2023 (европейский формат)
         ]
 
         for fmt in date_formats:
@@ -213,6 +274,7 @@ class Common:
 
     def analyze_column_types(self, data, uniq_columns, partitions, text_columns_set):
         try:
+
             null_columns = []
             column_types = {}
             # Проходим по всем строкам в данных
