@@ -6,6 +6,7 @@ from datetime import datetime,timedelta
 import clickhouse_connect
 import pandas as pd
 import os
+import csv
 from dateutil import parser
 import time
 import hashlib
@@ -15,9 +16,9 @@ from dateutil.relativedelta import relativedelta
 
 
 class OZONbyDate:
-    def __init__(self,  bot_token:str, chats:str, message_type: str, subd: str,
-                 host: str, port: str, username: str, password: str, database: str,
-                                  add_name: str, clientid:str, token: str ,  start: str, backfill_days: int, reports :str):
+    def __init__(self,  bot_token:str = '', chats:str = '', message_type: str = '', subd: str = '',
+                 host: str = '', port: str = '', username: str = '', password: str = '', database: str = '',
+                                  add_name: str = '', clientid:str = '', token: str  = '',  start: str = '', backfill_days: int = 0, reports :str = ''):
         self.bot_token = bot_token
         self.chat_list = chats.replace(' ', '').split(',')
         self.message_type = message_type
@@ -44,7 +45,7 @@ class OZONbyDate:
                 'report_name': 'transactions',
                 'upload_table': 'transactions',
                 'func_name': self.get_transactions,
-                'uniq_columns': 'operation_date, operation_id',
+                'uniq_columns': 'operation_date,operation_id',
                 'partitions': 'operation_date',
                 'merge_type': 'ReplacingMergeTree(timeStamp)',
                 'refresh_type': 'nothing',
@@ -65,6 +66,32 @@ class OZONbyDate:
                 'frequency': 'daily',  # '2dayOfMonth,Friday'
                 'delay': 30
             },
+            'stocks_sku': {
+                'platform': 'ozon',
+                'report_name': 'stocks_sku',
+                'upload_table': 'stocks_sku',
+                'func_name': self.get_stocks_sku,
+                'uniq_columns': 'sku',
+                'partitions': '',
+                'merge_type': 'MergeTree',
+                'refresh_type': 'delete_all',
+                'history': False,
+                'frequency': 'daily',  # '2dayOfMonth,Friday'
+                'delay': 30
+            },
+            'stocks_sku_history': {
+                'platform': 'ozon',
+                'report_name': 'stocks_sku_history',
+                'upload_table': 'stocks_sku_history',
+                'func_name': self.get_stocks_sku,
+                'uniq_columns': 'sku',
+                'partitions': '',
+                'merge_type': 'MergeTree',
+                'refresh_type': 'nothing',
+                'history': False,
+                'frequency': 'daily',  # '2dayOfMonth,Friday'
+                'delay': 30
+            },
             'stocks_history': {
                 'platform': 'ozon',
                 'report_name': 'stocks_history',
@@ -91,12 +118,12 @@ class OZONbyDate:
                 'frequency': 'daily',  # '2dayOfMonth,Friday'
                 'delay': 30
             },
-            'returns_fbo': {
+            'products_info': {
                 'platform': 'ozon',
-                'report_name': 'returns_fbo',
-                'upload_table': 'returns_fbo',
-                'func_name': self.get_all_returns_fbo,
-                'uniq_columns': 'return_id',
+                'report_name': 'products_info',
+                'upload_table': 'products_info',
+                'func_name': self.get_all_products_info,
+                'uniq_columns': 'id',
                 'partitions': '',
                 'merge_type': 'MergeTree',
                 'refresh_type': 'delete_all',
@@ -104,11 +131,11 @@ class OZONbyDate:
                 'frequency': 'daily',  # '2dayOfMonth,Friday'
                 'delay': 30
             },
-            'returns_fbs': {
+            'returns': {
                 'platform': 'ozon',
-                'report_name': 'returns_fbs',
-                'upload_table': 'returns_fbs',
-                'func_name': self.get_all_returns_fbs,
+                'report_name': 'returns',
+                'upload_table': 'returns',
+                'func_name': self.get_all_returns,
                 'uniq_columns': 'id',
                 'partitions': '',
                 'merge_type': 'MergeTree',
@@ -117,6 +144,19 @@ class OZONbyDate:
                 'frequency': 'daily',  # '2dayOfMonth,Friday'
                 'delay': 30
             },
+            'returns_days': {
+                'platform': 'ozon',
+                'report_name': 'returns_days',
+                'upload_table': 'returns_days',
+                'func_name': self.get_returns,
+                'uniq_columns': 'id,company_id,order_id,logistic_return_date',
+                'partitions': '',
+                'merge_type': 'ReplacingMergeTree(timeStamp)',
+                'refresh_type': 'nothing',
+                'history': True,
+                'frequency': 'daily',  # '2dayOfMonth,Friday'
+                'delay': 30
+            },
             'realization': {
                 'platform': 'ozon',
                 'report_name': 'realization',
@@ -130,6 +170,19 @@ class OZONbyDate:
                 'frequency': '6',  # '2,Friday'
                 'delay': 30
             },
+            'realization_posting': {
+                'platform': 'ozon',
+                'report_name': 'realization_posting',
+                'upload_table': 'realization_posting',
+                'func_name': self.get_realization_posting,
+                'uniq_columns': 'year_month,row_number',
+                'partitions': 'year_month',
+                'merge_type': 'ReplacingMergeTree(timeStamp)',
+                'refresh_type': 'nothing',
+                'history': True,
+                'frequency': '6',  # '2,Friday'
+                'delay': 30
+            },
             'postings_fbo': {
                 'platform': 'ozon',
                 'report_name': 'postings_fbo',
@@ -143,6 +196,19 @@ class OZONbyDate:
                 'frequency': 'daily',  # '2,Friday'
                 'delay': 30
             },
+            'postings_fbs_rep': {
+                'platform': 'ozon',
+                'report_name': 'postings_fbs_rep',
+                'upload_table': 'postings_fbs_rep',
+                'func_name': self.get_postings_fbs_report,
+                'uniq_columns': 'load_date',
+                'partitions': 'load_date',
+                'merge_type': 'MergeTree', # тут надо выбрать схему обновления
+                'refresh_type': 'delete_date',
+                'history': True,
+                'frequency': 'daily',  # '2,Friday'
+                'delay': 30
+            },
             'finance_details': {
                 'platform': 'ozon',
                 'report_name': 'finance_details',
@@ -171,6 +237,199 @@ class OZONbyDate:
             },
         }
 
+    def create_postings_report(self, date, report_type):
+        try:
+            url = "https://api-seller.ozon.ru/v1/report/postings/create"
+            headers = {
+                "Client-Id": self.clientid,
+                "Api-Key": self.token,
+                "Content-Type": "application/json"
+            }
+            payload = {
+                "filter": {
+                    "processed_at_from": f"{date}T00:00:00.000Z",
+                    "processed_at_to": f"{date}T23:59:59.999Z",
+                    "delivery_schema": [report_type]
+                },
+                "language": "RU"
+            }
+            response = requests.post(url, headers=headers, data=json.dumps(payload))
+            code = response.status_code
+            if code == 429:
+                self.err429 = True
+            if code == 200:
+                result = response.json()
+                report_code = result['result']['code']
+                message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: create_postings_report. Результат: ОК'
+                self.common.log_func(self.bot_token, self.chat_list, message, 1)
+                return report_code
+            else:
+                response.raise_for_status()
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: create_postings_report. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+
+    def create_products_report(self):
+        try:
+            url = "https://api-seller.ozon.ru/v1/report/products/create"
+            headers = {
+                "Client-Id": self.clientid,
+                "Api-Key": self.token,
+                "Content-Type": "application/json"
+            }
+            payload = {
+                "language": "RU",
+                "visibility": "ALL"
+                }
+            response = requests.post(url, headers=headers, data=json.dumps(payload))
+            code = response.status_code
+            if code == 429:
+                self.err429 = True
+            if code == 200:
+                result = response.json()
+                report_code = result['result']['code']
+                message = f'Платформа: OZON. Имя: {self.add_name}. Функция: create_products_report. Результат: ОК'
+                self.common.log_func(self.bot_token, self.chat_list, message, 1)
+                return report_code
+            else:
+                response.raise_for_status()
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: create_products_report. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return e
+
+
+    def get_report_info(self, report_code):
+        try:
+            url = "https://api-seller.ozon.ru/v1/report/info"
+            headers = {
+                "Client-Id": self.clientid,
+                "Api-Key": self.token,
+                "Content-Type": "application/json"
+            }
+            payload = {
+                "code": report_code
+            }
+            response = requests.post(url, headers=headers, data=json.dumps(payload))
+            code = response.status_code
+            if code == 429:
+                self.err429 = True
+            if code == 200:
+                result = response.json()
+                return result
+            else:
+                response.raise_for_status()
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: get_report_info. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+    def get_ozon_stocks(self, skus):
+        try:
+            url = "https://api-seller.ozon.ru/v1/analytics/stocks"
+            headers = {
+                "Content-Type": "application/json",
+                "Client-Id": self.clientid,
+                "Api-Key": self.token
+            }
+            batch_size = 100
+            all_results = []
+            for i in range(0, len(skus), batch_size):
+                batch = skus[i:i + batch_size]
+                batch_str = [str(x) for x in batch if x!=0 and x!='0']
+                if batch_str:
+                    payload = {"skus": batch_str}
+                    response = requests.post(url, headers=headers, json=payload)
+                    code = response.status_code
+                    if code == 429:
+                        self.err429 = True
+                    if code == 200:
+                        data = response.json()
+                        items = data.get("items", [])
+                        all_results.extend(items)
+                    else:
+                        response.raise_for_status()
+                    time.sleep(2)
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: get_ozon_stocks. Результат: ОК'
+            self.common.log_func(self.bot_token, self.chat_list, message, 1)
+            return  all_results
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: get_ozon_stocks. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+    def get_all_skus(self, data_list):
+        try:
+            sku_list = []
+            for k in data_list:
+                try:
+                    sku_list.append(k['SKU'])
+                except:
+                    pass
+            return sku_list
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: get_all_skus. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+
+    def csv_to_dict_list(self,url):
+        try:
+            response = requests.get(url)
+            response.raise_for_status()
+            csv_content = StringIO(response.text)
+            csv_reader = csv.reader(csv_content, delimiter=';')
+            headers = next(csv_reader, None)
+            result = [dict(zip(headers, row)) for row in csv_reader if row]
+            if not headers  or not result:
+                message = f'Платформа: OZON. Имя: {self.add_name}. Функция: csv_to_dict_list. ПУСТОЙ ОТЧЁТ.'
+                self.common.log_func(self.bot_token, self.chat_list, message, 2)
+            return result
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Функция: csv_to_dict_list. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+    def get_postings_fbs_report(self, date):
+        try:
+            new_report = self.create_postings_report(date,'fbs')
+            for k in range(20):
+                time.sleep(10)
+                get_link = self.get_report_info(new_report)
+                if get_link['result']['status'] == 'success':
+                    data = self.common.transliterate_dict_keys_in_list(self.csv_to_dict_list(get_link['result']['file']))
+                    for elem in data:
+                        elem['load_date'] = date
+                    return data
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {date}. Функция: get_postings_fbs_report. Результат: ОК.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 1)
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {date}. Функция: get_postings_fbs_report. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+
+    def get_stocks_sku(self, date=''):
+        try:
+            new_report = self.create_products_report()
+            for k in range(50):
+                time.sleep(10)
+                get_link = self.get_report_info(new_report)
+                if get_link['result']['status'] == 'success':
+                    data = self.csv_to_dict_list(get_link['result']['file'])
+                    skus = self.get_all_skus(data)
+                    result = self.get_ozon_stocks(skus)
+                    for elem in result:
+                        elem['load_date'] = date
+                    return result
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {date}. Функция: get_stocks_sku. Результат: ОК.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 1)
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {date}. Функция: get_stocks_sku. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
 
     def get_transaction_page_count(self, date):
         try:
@@ -238,7 +497,7 @@ class OZONbyDate:
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             return message
 
-    def get_stock_on_warehouses(self, date):
+    def get_stock_on_warehouses(self, date=''):
         try:
             url = "https://api-seller.ozon.ru/v2/analytics/stock_on_warehouses"
             headers = {
@@ -276,9 +535,9 @@ class OZONbyDate:
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             return message
 
-    def get_all_products(self, date):
+    def get_all_products(self, date=''):
         try:
-            url = "https://api-seller.ozon.ru/v2/product/list"
+            url = "https://api-seller.ozon.ru/v3/product/list"
             headers = {
                 "Client-Id": self.clientid,
                 "Api-Key": self.token,
@@ -290,7 +549,8 @@ class OZONbyDate:
             while True:
                 payload = {
                     "last_id": last_id,
-                    "limit": limit
+                    "limit": limit,
+                    "filter" : {}
                 }
                 response = requests.post(url, headers=headers, data=json.dumps(payload))
                 code = response.status_code
@@ -315,15 +575,69 @@ class OZONbyDate:
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             return message
 
-    def get_all_returns_fbo(self, date):
+    def get_all_products_info(self, date=''):
         try:
-            url = "https://api-seller.ozon.ru/v3/returns/company/fbo"
+            url = "https://api-seller.ozon.ru/v3/product/list"
+            url2 = "https://api-seller.ozon.ru/v3/product/info/list"
             headers = {
                 "Client-Id": self.clientid,
                 "Api-Key": self.token,
                 "Content-Type": "application/json"
             }
-            limit = 1000  # Можно задать желаемый лимит записей на один запрос
+            limit = 1000
+            last_id = ""  # Инициализируем last_id пустым значением для первого запроса
+            all_items = []  # Список для хранения всех продуктов
+            while True:
+                payload = {
+                    "last_id": last_id,
+                    "limit": limit,
+                    "filter" : {}
+                }
+                response = requests.post(url, headers=headers, data=json.dumps(payload))
+                code = response.status_code
+                if code == 429:
+                    self.err429 = True
+                if code == 200:
+                    result = response.json().get('result', {})
+                    items = result.get('items', [])
+                    if not items:
+                        break
+                    product_ids = []
+                    for item in items:
+                        product_ids.append(item['product_id'])
+                    payload2 = {"product_id" : product_ids}
+                    response2 = requests.post(url2, headers=headers, data=json.dumps(payload2))
+                    code2 = response2.status_code
+                    if code2 == 429:
+                        self.err429 = True
+                    if code2 == 200:
+                        items2 = response2.json().get('items', [])
+                        if not items2:
+                            break
+                    all_items.extend(items2)
+                    if len(items) < limit:
+                        break
+                    last_id = result.get('last_id', "")
+                else:
+                    response.raise_for_status()
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_products_info. Результат: ОК'
+            self.common.log_func(self.bot_token, self.chat_list, message, 1)
+            return self.common.spread_table(self.common.spread_table(all_items))
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_products_info. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
+
+    def get_all_returns(self, date=''):
+        try:
+            url = "https://api-seller.ozon.ru/v1/returns/list"
+            headers = {
+                "Client-Id": self.clientid,
+                "Api-Key": self.token,
+                "Content-Type": "application/json"
+            }
+            limit = 500  # Можно задать желаемый лимит записей на один запрос
             last_id = 0  # Инициализируем last_id с начальным значением 0
             all_returns = []  # Список для хранения всех возвратов
             while True:
@@ -343,32 +657,37 @@ class OZONbyDate:
                     all_returns.extend(returns)  # Добавляем все элементы 'returns' в общий список
                     if len(returns) < limit:
                         break
-                    last_id = result.get('last_id', 0)
+                    last_id = int(returns[-1]['id'])
                 else:
                     response.raise_for_status()
-            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbo. Результат: ОК'
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns. Результат: ОК'
             self.common.log_func(self.bot_token, self.chat_list, message, 1)
             return all_returns  # Возвращаем итоговый список из 'returns'
         except Exception as e:
-            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbo. Ошибка: {e}.'
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns. Ошибка: {e}.'
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             return message
 
-    def get_all_returns_fbs(self, date):
+    def get_returns(self, date):
         try:
-            url = "https://api-seller.ozon.ru/v3/returns/company/fbs"
+            url = "https://api-seller.ozon.ru/v1/returns/list"
             headers = {
                 "Client-Id": self.clientid,
                 "Api-Key": self.token,
                 "Content-Type": "application/json"
             }
-            limit = 1000  # Устанавливаем лимит на количество записей за запрос
+            limit = 500  # Можно задать желаемый лимит записей на один запрос
             last_id = 0  # Инициализируем last_id с начальным значением 0
             all_returns = []  # Список для хранения всех возвратов
             while True:
                 payload = {
-                    "limit": limit,
-                    "last_id": last_id
+                    "filter": {
+                        "logistic_return_date":
+                            {                            "time_from": f"{date}T00:00:00Z",
+                            "time_to": f"{date}T23:59:59Z"                        }
+                    },
+                    "last_id": last_id,
+                    "limit": limit
                 }
                 response = requests.post(url, headers=headers, data=json.dumps(payload))
                 code = response.status_code
@@ -379,20 +698,24 @@ class OZONbyDate:
                     returns = result.get('returns', [])
                     if not returns:
                         break
-                    all_returns.extend(returns)
+                    all_returns.extend(returns)  # Добавляем все элементы 'returns' в общий список
                     if len(returns) < limit:
                         break
-                    last_id = result.get('last_id', 0)
+                    last_id = int(returns[-1]['id'])
                 else:
                     response.raise_for_status()
-            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbs. Результат: ОК'
+            all_returns = self.common.spread_table(self.common.spread_table(all_returns))
+            for dict in all_returns:
+                dict['logistic_return_date'] = date
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_returns. Результат: ОК'
             self.common.log_func(self.bot_token, self.chat_list, message, 1)
-            return all_returns
+            return all_returns  # Возвращаем итоговый список из 'returns'
         except Exception as e:
-            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbs. Ошибка: {e}.'
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_returns. Ошибка: {e}.'
             self.common.log_func(self.bot_token, self.chat_list, message, 3)
             return message
 
+
     def get_realization(self, date):
         try:
             real_date = datetime.strptime(date, "%Y-%m-%d")
@@ -432,6 +755,44 @@ class OZONbyDate:
             return message
 
 
+    def get_realization_posting(self, date):
+        try:
+            real_date = datetime.strptime(date, "%Y-%m-%d")
+            last_month_date = real_date - relativedelta(months=1)
+            previous_month = last_month_date.month
+            previous_year = last_month_date.year
+            yyyy_mm = f"{previous_year}-{str(previous_month).zfill(2)}-01"
+            final_data = []
+            data = {
+                "month": previous_month,
+                "year": previous_year
+            }
+            headers = {
+                "Client-Id": self.clientid,
+                "Api-Key": self.token,
+                "Content-Type": "application/json"
+            }
+            url = "https://api-seller.ozon.ru/v1/finance/realization/posting"
+            response = requests.post(url, json=data, headers=headers)
+            code = response.status_code
+            if code == 429:
+                self.err429 = True
+            if code == 200:
+                result = response.json().get('rows', [])
+                for row in result:
+                    row['year_month'] = yyyy_mm
+                    final_data.append(row)
+                final_result = self.common.spread_table(final_data)
+            else:
+                response.raise_for_status()
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realization_posting. Результат: ОК'
+            self.common.log_func(self.bot_token, self.chat_list, message, 1)
+            return final_result
+        except Exception as e:
+            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realization_posting. Ошибка: {e}.'
+            self.common.log_func(self.bot_token, self.chat_list, message, 3)
+            return message
+
     def get_postings_fbo(self, date):
         try:
             url = "https://api-seller.ozon.ru/v2/posting/fbo/list"
@@ -486,7 +847,7 @@ class OZONbyDate:
         try:
             date_obj = datetime.strptime(date, "%Y-%m-%d")
             day = date_obj.day
-            if day == 1:
+            if day <= 15:
                 # С 16-го числа предыдущего месяца по конец предыдущего месяца
                 last_day_of_prev_month = date_obj.replace(day=1) - timedelta(days=1)
                 start_date = last_day_of_prev_month.replace(day=16)
@@ -633,6 +994,7 @@ class OZONbyDate:
                     self.source_dict[report]['frequency'],
                     self.source_dict[report]['delay']
                 )
+        self.common.send_logs_clear_anyway(self.bot_token, self.chat_list)
 
 
 
